/****************************************************************************
 *
 * BPELModelFactory.hh
 *
 * This file is part of bpel2qnbound
 *
 * Copyright (C) 2007, 2018 Moreno Marzolla
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#ifndef BPEL_BPELMODELFACTORY_HH
#define BPEL_BPELMODELFACTORY_HH

#include <map>
#include <string>
#include <set>
#include <libxml/tree.h>

class Activity;
class BPELModel;
class WSDLOperation;

class BPELModelFactory {
protected:
    /**
     * Builds an invoke activity from an XML node
     */ 
    Activity* make_invoke( xmlNodePtr n );

    /**
     * Builds a sequence activity from an XML node
     */
    Activity* make_sequence( xmlNodePtr n );

    /**
     * Builds a flow activity from an XML node
     */
    Activity* make_flow( xmlNodePtr n );

    /**
     * Builds a while activity from an XML node
     */
    Activity* make_while( xmlNodePtr n );

    /**
     * Builds a repeatUntil activity from an XML node
     */
    Activity* make_repeat( xmlNodePtr n );

    /**
     * Makes a dummy activity
     */
    Activity* make_dummy( xmlNodePtr n );

    /**
     * Builds an activity from xml node n.
     *
     * @param n the xml node from which the activity is to be built
     *
     * @return the created activity (0 if creation failed). The caller
     * owns the returned pointer.
     */ 
    Activity* make_activity( xmlNodePtr n );

    /**
     * Parses a WSDL specification, looking for "operation" specifications
     */
    void parse_wsdl( const std::string& fname );

    /**
     * Builds a BPEL Model from its XML specification. The caller owns
     * the returned object.
     */
    void make_bpel_model( xmlNodePtr n );

    typedef Activity* ( BPELModelFactory::*xml_handler)( xmlNodePtr );
    std::map< std::string, xml_handler > m_handler;
    std::map< std::string, WSDLOperation* > m_operations;
    std::set< std::string > m_resources;
    BPELModel* m_bpel_model;

public:
    /**
     * Initializes the factory with a given bpel specification
     *
     * @param fname the BPEL file name
     */ 
    BPELModelFactory( const std::string& fname );

    ~BPELModelFactory( ) { };

    /**
     * Gets the BPEL model which has been built
     */
    BPELModel* get_bpel_model( void ) { return m_bpel_model; };
};

#endif
