/****************************************************************************
 *
 * ServiceDemandVisitor.hh
 *
 * This file is part of bpel2qnbound
 *
 * Copyright (C) 2007, 2018 Moreno Marzolla
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#ifndef BPEL_SERVICEDEMANDVISITOR_HH
#define BPEL_SERVICEDEMANDVISITOR_HH

#include "ActivityVisitor.hh"
#include <map>
#include <string>

/**
 * This class is used to compute the service demand on resources of a
 * BPEL model. Instead of computing the visit ratios using the
 * equations described in the paper, we adopt a simpler approach
 * (which however would not work unless there are no "gotos" in the
 * BPEL model) by recursively computing the visit ratios by structural
 * induction. This approach produces exactly the same results than the
 * one described in the paper, but is less general.
 *
 * Invoke this class by using:
 *
 * BPELModel* m = ... ;
 * ServiceDemandVisitor v( );
 * m->accept( v );
 * v.describe();
 */
class ServiceDemandVisitor : public ActivityVisitor {
protected:
    double m_outer_visit_count;
    std::map< std::string, double > m_demand;
    std::map< std::string, double > m_visits;
    double m_D_max; // Max service demand
    double m_D; // Sum of demands
    double m_D_ave; // Average demand
    BPELModel* m_model;

    void describe_open( const std::string& fname ) const;
    void describe_closed( const std::string& fname ) const;

public:
    ServiceDemandVisitor( );

    virtual ~ServiceDemandVisitor( ) { };

    void describe( const std::string& fname ) const;

    void visit_sequence( SequenceActivity* a );
    void visit_invoke( InvokeActivity* a );
    void visit_flow( FlowActivity* a );
    void visit_dummy( DummyActivity* a );
    void visit_while( WhileActivity* a );
    void visit_repeat( RepeatActivity* a );
    void visit_bpel_model( BPELModel* m );
};

#endif
