/****************************************************************************
 *
 * WSDLFactory.cc
 *
 * This file is part of bpel2qnbound
 *
 * Copyright (C) 2007, 2018 Moreno Marzolla
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#include "WSDLFactory.hh"
#include "WSDLOperation.hh"
#include "Utils.hh"

#include <iostream>
#include <cassert>
#include <cstdlib>

#include <libxml/xpath.h>
#include <libxml/tree.h>
#include <libxml/xpathInternals.h>

using namespace std;

WSDLFactory::WSDLFactory( const string& fname )
{
    xmlDocPtr doc;

    xmlSubstituteEntitiesDefault( 1 );
    doc = xmlParseFile( fname.c_str() );
    if ( 0 == doc ) {
        cerr << "FATAL: An error occurred while parsing WSDL '" 
             << fname << "'" << endl;
        abort( );
    }
    xmlDocGetRootElement( doc );
    parse_wsdl( doc );
    xmlFreeDoc( doc );
}

WSDLFactory::~WSDLFactory( )
{
}

WSDLOperation* WSDLFactory::make_operation( xmlNodePtr n )
{
    assert( 0 != n );
    WSDLOperation* result = new WSDLOperation( BPELAttribute( n, "name" ).as_string() );
    // Parses the child nodes, looking for perf annotations
    for ( xmlNodePtr tmp = n->children; 0 != tmp; tmp = tmp->next ) {
        if ( 0 == xmlStrcmp( tmp->name, (xmlChar*)"PAdemand" ) ) {
            string res_name( BPELAttribute( tmp, "resource" ).as_string() );
            double serv_time( BPELAttribute( tmp, "value" ).as_double() );
            result->set_service_time( res_name, serv_time );
        }
    }
    return result;
}

void WSDLFactory::parse_wsdl( xmlDocPtr doc )
{
    xmlXPathContextPtr xpathCtx; 
    xmlXPathObjectPtr xpathObj; 
    xpathCtx = xmlXPathNewContext(doc);
    // register the wsdl namespace
    xmlXPathRegisterNs(xpathCtx, (xmlChar*)"wsdl", (xmlChar*)"http://schemas.xmlsoap.org/wsdl/");
    const char* xpathExpr = "//wsdl:operation";

    xpathObj = xmlXPathEvalExpression((xmlChar*)xpathExpr, xpathCtx);
    xmlNodeSetPtr nodes = xpathObj->nodesetval;

    for ( int i=0; i<nodes->nodeNr; ++i ) {
        if ( nodes->nodeTab[i]->type == XML_ELEMENT_NODE ) {
            WSDLOperation* op = make_operation( nodes->nodeTab[i] );
            if ( op ) {
                op->describe();
                m_operations.push_back( op );
            }
        }
    }

    xmlXPathFreeObject(xpathObj);
    xmlXPathFreeContext(xpathCtx); 
}

list< WSDLOperation* > WSDLFactory::get_operations( void ) const
{
    return m_operations;
}
